#! perl -w

# Licensed Materials - Property of IBM                       
# Component IBM Rational ROSE_RT BUILD : TARGET_RTS                 
# Copyright IBM Corporation 1999, 2008. All Rights Reserved.      
# U.S. Government Users Restricted Rights -  Use, duplication or    
# disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.



# rtdep.pl - Dependency finder for header files only (for model interface)

# Force autoflush for nice error messages

select STDERR; $| = 1;
select STDOUT; $| = 1;

# Get RTS_HOME and load in rterror

$rtsHome = $0;
$rtsHome =~ s:\\:/:g;
$rtsHome =~ s:/codegen/rtdeps.pl$::;

# load in the error handler

require "$rtsHome/codegen/rterror.pl";

# Silence perl: other uses are in rterror.pl.
$rterror::maxError = $rterror::maxError;

# Get the arguments

while( $#ARGV >= 0 )
{
    $arg = shift @ARGV;
    if   ( $arg eq '-I'   ) { push @deppath,   shift @ARGV; }
    elsif( $arg eq '-src' ) { $srcobject     = shift @ARGV;
			      $srcfile       = shift @ARGV; }
    else
    {
	&rterror::print_error( '', "Unknown rtcomp.pl option $arg", 1 );
    }
}

####
#### MAKEDEPEND
####

sub find_file($)
{
    my( $file ) = @_;

    if( -e $file )
    {
	# a special case: has a complete path already been provided?
	return $file;
    }

    foreach $loc ( @deppath )
    {
	if( -e "$loc/$file" )
	{
	    return "$loc/$file";
	}
    }

    return '';
}

# srcfile is first, let's make all the slashes go one way
$srcfile =~ s:\\:/:g;
@deplist = ( $srcfile );
$deps{$srcfile} = 1;

foreach $file ( @deplist )
{
    if( ! open FILE, "$file" )
    {
	&rterror::print_error( '', "Could not open [$file]", 1 );
	exit 1;
    }

    while( <FILE> )
    {
	# recognize: include <X> or include "X"
	if( m/^\s*#\s*include\s*\<([^\>]*)\>.*$/o or
	    m/^\s*#\s*include\s*\"([^\"]*)\".*$/o )
	{
	    my( $depfile ) = find_file( $1 );

	    if( $depfile && ! $deps{$depfile} )
	    {
		$deps{$depfile} = 1;
		push @deplist, $depfile;
	    }
	}
    }

    close FILE;
}

$depfile = "$srcobject.dep";

if( ! open DEPFILE, ">$depfile" )
{
    &rterror::print_error( '', "Cannot write to '$depfile'", 0 );
    exit 1;
}

binmode( DEPFILE );

print DEPFILE $srcobject, "_DEPS =";

foreach $dep ( @deplist )
{
    if( $dep =~ /\s/ )
    {
	$dep = '"' . $dep . '"';
    }

    print DEPFILE " \\\n\t$dep";
}

print DEPFILE "\n";

close DEPFILE;

if( $rterror::worstError < $rterror::maxError )
{
    if( $rterror::worstError == 0 )
    {
	exit 1;
    }
}
