/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTActor_class.h>
#include <RTBindingEnd.h>
#include <RTDaemonInfo.h>
#include <RTDebugger.h>
#include <RTMemoryUtil.h>
#include <RTPortDescriptor.h>
#include <RTProtocol.h>

int RTActorProbe::inject( int                    inwards,
			  const char           * signalName,
			  int                    prio,
			  const void           * data,
			  const RTObject_class * type,
			  const RTDaemonInfo   & daemonInfo )
{
    if( _actor == (RTActor *)0 )
	return 0;

    const RTActor_class    * info      = _actor->getActorData();
    const RTPortDescriptor * portDesc  = info->ports;
    RTProtocol             * port;
    int                      portIndex = 0;

    for( portIndex = 0; ; ++portIndex, ++portDesc )
    {
	if( portIndex >= info->numPorts )
	    return 0;
	if( RTMemoryUtil::strcmp( portDesc->name, daemonInfo.portName ) == 0 )
	{
	    port = RTObjectOf( RTProtocol, portDesc->offset, _actor );
	    break;
	}
    }

    int portSize = port->size();

    if( daemonInfo.portIndex >= portSize )
	return 0;

    if( daemonInfo.portIndex < 0 )
	portIndex = 0;
    else
    {
	portIndex = daemonInfo.portIndex;
	portSize  = portIndex + 1;
    }

    if( inwards != 0 )
    {
	int signal = port->getInfo()->protocol->inSignalNamed( signalName );

	if( signal <= 0 )
	    return 0;

	for( ; portIndex < portSize; ++portIndex )
	    if( _debugger->doInject( port, portIndex,
				     signal, prio, data, type ) == 0 )
		return 0;
    }
    else
    {
	RTBindingEnd end;

	for( ; portIndex < portSize; ++portIndex )
	    if( port->peerAt( portIndex, end ) != 0 )
	    {
		int signal = end.port->getInfo()
				     ->protocol->inSignalNamed( signalName );

		if( signal <= 0 )
		    return 0;

		if( _debugger->doInject( end.port, end.index,
					 signal, prio, data, type ) == 0 )
		    return 0;
	    }
    }

    return 1;
}
