/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorClass.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTController.h>
#include <RTImportList.h>

int RTActorRef::import( RTActor * actor, int index )
{
    RTActor      * myOwner    = getOwner();
    RTController * controller = myOwner->context();

    controller->trace( RTController::FrameImport, this, &index );

    if( actor == (RTActor *)0 )
    {
	controller->setError( RTController::badActor,
			      RTController::FrameImport, this, &index );
	return 0;
    }

    if( _info->kind != RTComponentDescriptor::Imported )
    {
	controller->setError( RTController::notImported,
			      RTController::FrameImport, this, &index );
	return 0;
    }

    RTController::Error error;

    error = controller->canPlugIn( actor->getActorData()->type, _info );

    if( error != RTController::ok )
    {
	controller->setError( error, RTController::FrameImport, this, &index );

#if RTTYPECHECK_PROTOCOL == RTTYPECHECK_WARN
	if( error != RTController::protocolType )
#endif
	{
	    return 0;
	}
    }

    if( myOwner->isReferencedBy( actor ) != 0 )
    {
	// There is already a reference chain from actor to myOwner;
	// if I were to allow this import operation, we would have
	// a cycle in the reference graph.
	controller->setError( RTController::badClass,
			      RTController::FrameImport, this, &index );
	return 0;
    }

    error = index < 0 ? RTController::refFull
		      : RTController::badIndex;

    if( reserve( index ) == 0 )
    {
	controller->setError( error, RTController::FrameImport, this, &index );
	return 0;
    }

    int            replication = _info->replication;
    RTImportList * entry;

    if( replication == 1 )
	entry = (RTImportList *)_contents;
    else
    {
	RTImportList::Unused * head = (RTImportList::Unused *)_contents;

	entry = (RTImportList *)(void *)&head[ 1 ];
	entry += index;
    }

#if USE_THREADS && RTFRAME_THREAD_SAFE
    lock();
#endif

    add( actor, index );

    // add this reference to the list of places that actor has been imported

    entry->used.next  = actor->importList;
    actor->importList = entry;

    if( bind_ports( actor, index, this ) == 0 )
    {
	actor->importList = entry->used.next;
#if USE_THREADS && RTFRAME_THREAD_SAFE
	unlock();
#endif
	controller->setError( RTController::badClass,
			      RTController::FrameImport, this, &index );
	remove( index );
	return 0;
    }

#if USE_THREADS && RTFRAME_THREAD_SAFE
    unlock();
#endif

    return 1;
}
