/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActorRefProbe.h>
#include <RTActor_class.h>
#include <RTComponentDescriptor.h>
#include <RTMain.h>
#include <RTMemoryUtil.h>
#include <RTString.h>

RTActorRef * RTActorRefProbe::getRefFromStrPath( void )
{
    if( _stringPath == (char *)0 )
	return (RTActorRef *)0;

    RTActor * theActor = RTMain::topActor();

    if( theActor == (RTActor *)0 )
	return (RTActorRef *)0;

    const char * _actorPath = _stringPath; // for now

    if( _actorPath[ 0 ] != '/' ) // must start with "/"
	return (RTActorRef *)0;

    if( _actorPath[ 1 ] == '\0' ) // must be more than 1 character
	return (RTActorRef *)0;

    char * actorPath = RTMemoryUtil::strdup( &_actorPath[ 1 ] );
    char * slidePtr  = actorPath;
    char * actorPtr  = actorPath;
    char * replPtr   = (char *)0;

    for(;;)
    {
	if( *slidePtr == ':' )
	{
	    if( replPtr != (char *)0 ) // it _should_ be null
	    {
		delete [] actorPath;
		return (RTActorRef *)0;
	    }

	    *slidePtr++ = '\0';
	    replPtr = slidePtr;
	}

	if( *slidePtr == '/' || *slidePtr == '\0' )
	{
	    int  repIndex;
	    char saveChar;

	    saveChar  = *slidePtr;
	    *slidePtr = '\0';

	    if( replPtr != (char *)0 )
		repIndex = RTMemoryUtil::atoi( replPtr );
	    else if( saveChar == '\0' )
		repIndex = _actorIndex;
	    else
	    {
		delete [] actorPath;
		return (RTActorRef *)0;
	    }

	    const RTActor_class * info = theActor->getActorData();

	    for( int iComp = 0; iComp < info->numComponents; ++iComp )
	    {
		const RTComponentDescriptor * desc = &info->components[ iComp ];

		if( RTMemoryUtil::strcmp( desc->name, actorPtr ) == 0 )
		{
		    RTActorRef * theActorRef = RTObjectOf( RTActorRef,
							   desc->offset,
							   theActor );

		    if( repIndex < 0 ) // success since repIndex is -1 for refs
		    {
			delete [] actorPath;
			return theActorRef;
		    }

		    if( repIndex >= desc->replication ||
			(theActor = (*theActorRef)[repIndex]) == (RTActor *)0 )
		    {
			delete [] actorPath;
			return (RTActorRef *)0;
		    }

		    break;
		}
	    }

	    if( saveChar == '\0' )
		break;

	    actorPtr = slidePtr + 1;
	    replPtr  = (char *)0;
	}

	++slidePtr;
    }

    delete [] actorPath;

    return (RTActorRef *)0;
}
