/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTPrefix.h"
#pragma implementation "RTPrefix.inl"
#endif

#include <RTActor.h>
#include <RTActorRef.h>
#include <RTActor_class.h>
#include <RTDebugger.h>
#include <RTDiag.h>
#include <RTDiagStream.h>
#include <RTPrefix.h>

static void put_prefix( RTDiagStream & out, const RTPrefix & prefix )
{
    if( prefix.prefix != (RTPrefix *)0 )
    {
	put_prefix( out, *prefix.prefix );
	out << "/" << prefix.compId;

	if( prefix.index != 0 )
	    out << "." << ( prefix.index + 1 );
    }
}

static void describeState( RTDiagStream        & out,
			   const RTActor_class * info,
			   int                   state )
{
    const char * stateName = "?";

    if( 0 < state && state <= info->numStates )
    {
	if( --state != 0 )
	    describeState( out, info, (int)info->parentState[ state ] );

	stateName = info->names[ state ];
    }

    out << ":" << stateName;
}

static void describeActor( const RTActorRef & ref,
			   const RTActor    & actor,
			   const RTPrefix   & prefix,
			   const char       * path,
			   int                more )
{
    RTDiagStream & out = RTDiag::out();

    for( const RTPrefix * p  = prefix.prefix;
			  p != (const RTPrefix *)0;
			  p  = p->prefix )
	out << ". ";

    out << ref.getName() << " : " << actor.getTypeName();

    if( ref.isFixed() != 0 )
	out << "(fixed) <";
    else if( ref.isOptional() != 0 )
	out << "(optional) <";
    else if( ref.isImported() != 0 )
	out << "(imported) <";

    describeState( out, actor.getActorData(), actor.getCurrentState() );
    out << "> ";

    out << path;
    put_prefix( out, prefix );

    if( more > 0 )
	out << " [" << more << " more]";

    out << "\n";
}

static void showStructure( RTActorRef * ref,
			   RTActor    * actor,
			   RTPrefix   * context,
			   const char * path,
			   int          depth )
{
    int compCount = actor->_numComponents();
    int more      = 0;

    if( depth == 0 )
    {
	for( int compId = 1; compId <= compCount; ++compId )
	{
	    const RTActorRef * compRef = actor->_component( compId );

	    for( int index = 0; index < compRef->size(); ++index )
	    {
		if( (*compRef)[ index ] != (RTActor *)0 )
		    ++more;
	    }
	}
    }

    describeActor( *ref, *actor, *context, path, more );

    if( ref->isImported() != 0 )
	return; // do not expand imported actors

    if( depth == 0 )
	return; // do not go any deeper

    RTPrefix comp( context );
    for( comp.compId = 1; comp.compId <= compCount; ++comp.compId )
    {
	RTActorRef * compRef = actor->_component( comp.compId );

	for( comp.index = 0; comp.index < compRef->size(); ++comp.index )
	{
	    RTActor * compActor = (*compRef)[ comp.index ];
	    if( compActor != (RTActor *)0 )
		showStructure( compRef, compActor, &comp, path, depth - 1 );
	}
    }
}

void RTDebugger::printStructure( RTActor    * actor,
				 const char * path,
				 int          depth )
{
    if( actor == (RTActor *)0 )
    {
	RTDiag::out() << "No such actor\n";
	return;
    }

    RTPrefix context( (RTPrefix *)0 );

    showStructure( actor->getReference(), actor, &context, path, depth - 1 );
}
