/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDictionary.h>
#include <RTMemoryUtil.h>

int RTDictionary::atPut( const char * key, void * value )
{
    if( key == (const char *)0 )
	return 0; // keys must be non-nil

    unsigned        index   = hash( key, _size );
    RTAssociation * assoc   = &_contents[ index ];
    unsigned        tests   = 0;
    RTAssociation * deleted = (RTAssociation *)0;

    for(;;)
    {
	if( assoc->key == (char *)0 )
	{
	    // this slot has never been used
	    if( deleted != (RTAssociation *)0 )
		assoc = deleted; // use a deleted entry, if we saw one
	    break;
	}

	if( assoc->key == (char *)assoc )
	{
	    if( deleted == (RTAssociation *)0 )
		deleted = assoc;
	}
	else if( RTMemoryUtil::strcmp( assoc->key, key ) == 0 )
	    return 0; // the key already exists

	if( ++tests == _size )
	{
	    // we looked everywhere; use the first deleted entry, if any
	    assoc = deleted;
	    break;
	}

	if( ++index == _size )
	{
	    index = 0;
	    assoc = _contents;
	}
	else
	    ++assoc;
    }

    if( assoc == (RTAssociation *)0 )
    {
	// the table needs to be expanded

	unsigned        old_size     = _size;
	RTAssociation * old_contents = _contents;

	_size   <<= 1;
	_used     = 0;
	_contents = new RTAssociation [ _size ];

	for( unsigned i = 0; i < _size; ++i )
	    _contents[ i ].key = (char *)0;

	for( assoc = old_contents; old_size-- != 0U; ++assoc )
	{
	    (void)atPut( assoc->key, assoc->value );
	    delete [] assoc->key;
	}
	(void)atPut( key, value );

	delete [] old_contents;
    }
    else
    {
	assoc->key   = RTMemoryUtil::strdup( key );
	assoc->value = value;
	++_used;
    }

    return 1;
}
