/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDebugger.h>
#include <RTMemoryUtil.h>
#include <RTObserver.h>
#include <RTTcpSocket.h>

void RTObserver::cmdCommand( const char * commandString )
{
    // assume this is not a flow cmd (pause/resume will change this)
    lastFlowCmd = FlowNone;

    // run, step & stop are 'user' commands which all affect the
    // resumeToRun flag
    if(      0 == RTMemoryUtil::strcmp( commandString, "run"    ) )
    {
	haltByProbe = 0;
	resumeToRun = 1;
	debugger->proceed();
    }

    else if( 0 == RTMemoryUtil::strcmp( commandString, "step"   ) )
    {
	haltByProbe = 0;
	resumeToRun = 0;
	debugger->step( 1U );
    }

    else if( 0 == RTMemoryUtil::strcmp( commandString, "stop"   ) )
    {
	haltByProbe = 0;
	resumeToRun = 0;
	debugger->step( 0U );
    }

    // pause is a 'system' command which doesn't affect the resumeToRun flag
    else if( 0 == RTMemoryUtil::strcmp( commandString, "pause"  ) )
    {
	lastFlowCmd = FlowPause;
	debugger->step( 0U );
    }

    // resume only runs if the last user command was run
    // AND execution was not halted by probe
    else if( 0 == RTMemoryUtil::strcmp( commandString, "resume" ) )
    {
	lastFlowCmd = FlowResume;
	if( resumeToRun != 0 && haltByProbe == 0 )
	    debugger->proceed();
    }

    // exit terminates the executable, after closing down all the sockets
    else if( 0 == RTMemoryUtil::strcmp( commandString, "exit"   ) )
    {
	debugger->doCmdExit();
    }
}
