/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTController.h>
#include <RTDebugger.h>
#include <RTEventInfo.h>
#include <RTMessage.h>
#include <RTProtocol.h>

int RTProtocol::raise( int                    signal,
		       const void           * data,
		       const RTObject_class * type )
{
    RTActor      * owner      = getOwner();
    RTController * controller = owner->context();

    controller->trace( RTController::ExceptionRaise, this, (const void *)0 );

#if RTTYPECHECK_SEND != RTTYPECHECK_DONT
    switch( sendCheck( signal, type ) )
    {
    case RTProtocolDescriptor::badSignal:
	controller->setError( RTController::badSignal,
			      RTController::ExceptionRaise,
			      this,
			      (const void *)0 );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::badType:
	controller->setError( RTController::badClass,
			      RTController::ExceptionRaise,
			      this,
			      (const void *)0 );
#if RTTYPECHECK_SEND == RTTYPECHECK_FAIL
	return 0;
#else
	break;
#endif

    case RTProtocolDescriptor::ok:
    default:
	break;
    }
#endif // RTTYPECHECK_SEND != RTTYPECHECK_DONT

    RTMessage * msg = controller->newMsg();

    if( msg == (RTMessage *)0 )
    {
	controller->setError( RTController::noMem,
			      RTController::ExceptionRaise,
			      this,
			      (const void *)0 );
	return 0;
    }

    msg->setData( data, type );
    msg->setSignal( signal );
    msg->setPriority( System );
    msg->setDestination( owner, this, 0 );
    msg->internData();

#if OBSERVABLE
    controller->debugger()->setTimestamp( msg, this, 0 );

    if( owner->probe != (RTActorProbe *)0 )
	owner->probe->evtMessageOut( msg, RTEventInfo::SendNormal );
#endif

    controller->receive( msg, controller );

    return 1;
}
