/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTController.h>
#include <RTTimerActor.h>
#include <RTTimerNode.h>
#include <RTTiming.h>

#if OBSERVABLE
#include <RTActor.h>
#include <RTActorProbe.h>
#include <RTDebugger.h>
#endif

RTTimerNode * Timing::Base::informAt( const RTTimespec     & when,
				      const void           * data,
				      const RTObject_class * type,
				      int                    prio )
{
    RTController * controller = context();
    RTTimerNode  * node       = (RTTimerNode *)0;
    RTTimerActor * timerActor;
    RTMessage    * msg;

    controller->trace( RTController::TimerInform, this, (const void *)0 );

    if( RTPriority::isValid( prio ) == 0 )
    {
	controller->setError( RTController::prio,
			      RTController::TimerInform,
			      this,
			      (const void *)0 );
    }
    else if( (timerActor = RTTimerActor::soleInstance()) == (RTTimerActor *)0 )
    {
	controller->setError( RTController::noConnect,
			      RTController::TimerInform,
			      this,
			      (const void *)0 );
    }
    else if( (msg = controller->newMsg()) == (RTMessage *)0 )
    {
	controller->setError( RTController::noMem,
			      RTController::TimerInform,
			      this,
			      (const void *)0 );
    }
    else if( (node = controller->newTimerNode()) == (RTTimerNode *)0 )
    {
	controller->setError( RTController::noMem,
			      RTController::TimerInform,
			      this,
			      (const void *)0 );
	controller->freeMsg( msg );
    }
    else
    {
	node->setValue( msg, this, when, RTTimespec::zero,
			data, type, prio );

	timerActor->informAt( node );

#if OBSERVABLE
	RTActor      * owner = getOwner();
	RTActorProbe * probe = owner->probe;

	if( probe != (RTActorProbe *)0 )
	{
	    RTMessage obsMsg;

	    obsMsg.setData( &when, &RTType_RTTimespec );

	    controller->debugger()->setTimestamp( &obsMsg, this, 0 );

	    probe->evtMessageOutIn( &obsMsg, RTEventInfo::SendInformAt );
	}
#endif
    }

    return node;
}
