/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#include <RTDiag.h>
#include <RTSyncObject.h>
#include <RTTimespec.h>

#include <errno.h>

void RTSyncObject::timedwait( const RTTimespec & expireTime )
{
    RTTimespec now;

    RTTimespec::getclock( now );

    if( expireTime <= now )
	return;

    enter();

    if( signalled == 0 )
    {
	// the timeout argument to pthread_cond_timedwait is 'timespec *'
	// instead of 'RTTimespec *' so we must make a copy
	timespec timeout;

	// In Lynx 3.0 and earlier the timeout times are relative,
	// but in 3.1 they are absolute.
#if TARGET_LYNX < 31
	RTTimespec expire( expireTime );
	expire -= now;

	timeout.tv_sec  = expire.tv_sec;
	timeout.tv_nsec = expire.tv_nsec;
#else
	timeout.tv_sec  = expireTime.tv_sec;
	timeout.tv_nsec = expireTime.tv_nsec;
#endif

	int rc = pthread_cond_timedwait( &cv, &mutex, &timeout );

	// POSIX, it seems, is a collection of 'standards'. Some implementions
	// use the return value to indicate the reason the call failed, others
	// simply return -1 and store the reason in errno. The following works
	// with all implementations encountered thus far.

	if( rc == 0 )
	{
	    // cv was signalled
	}
	else if( rc == EINTR     || errno == EINTR     ||
		 rc == ETIMEDOUT || errno == ETIMEDOUT )
	{
	    // timeout expired
	}
	else
	{
	    RTDiag::panic( "pthread_cond_timedwait" );
	}
    }

    signalled = 0;

    leave();
}
