/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTMemoryUtil.h>
#include <RTNucleus.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

static void thread_run( UNSIGNED, void * arg )
{
    RTThreadInfo * info = (RTThreadInfo *)arg;

    info->job->mainLoop();

    info->done = 1;
    info->sync.signal();
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job (                   job   )
    , _info( new RTThreadInfo( job ) )
{
    if( stack_size < NU_MIN_STACK_SIZE )
	stack_size = NU_MIN_STACK_SIZE;

    _info->stack = new char [ (unsigned)stack_size ];

    RTNucleus::Name name;

    RTNucleus::makeName( name, RTNucleus::Thread, this );

    if( NU_Create_Task( &_info->task,
			name,
			thread_run, 1U, _info,
			_info->stack,
			stack_size,
			(OPTION)prio,
			0U,           // no time slice
			NU_PREEMPT,
			NU_START ) != NU_SUCCESS )
	RTDiag::panic( "create(thread)" );
}

RTThreadInfo::RTThreadInfo( RTJob * _job )
    : job  (      _job )
    , stack( (char *)0 )
    , done (         0 )
    , sync (           )
{
#ifndef NU_NO_ERROR_CHECKING
    RTMemoryUtil::memset( &task, 0, (int)sizeof( task ) );
#endif
}

RTThreadInfo::~RTThreadInfo( void )
{
    while( done == 0 )
	sync.wait();

    if( NU_Terminate_Task( &task ) != NU_SUCCESS )
	RTDiag::panic( "terminate(thread)" );
    else if( NU_Delete_Task( &task ) != NU_SUCCESS )
	RTDiag::panic( "delete(thread)" );

    delete [] stack;
}
