/*
 * Licensed Materials - Property of IBM                               
 * Component IBM Rational ROSE_RT BUILD : TARGET_RTS                  
 * Copyright IBM Corporation 1999, 2008. All Rights Reserved.       
 * U.S. Government Users Restricted Rights -  Use, duplication or     
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTSyncObject.h>
#include <RTThread.h>
#include <RTThreadInfo.h>
#include <RTVrtx.h>

#include <vrtxil.h>

static void thread_run( void * arg )
{
    RTThreadInfo * info = (RTThreadInfo *)arg;

    info->job->mainLoop();

    // disable context switching so info doesn't disappear
    // before I've had a chance to signal
    sc_lock();

    info->done = 1;
    info->sync.signal();

    sc_unlock();

    for(;;)
    {
	int err;

	sc_tsuspend( 0, 0, &err );

	if( err != RET_OK )
	    RTDiag::panic( "sc_tsuspend" );
    }
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job (                   job   )
    , _info( new RTThreadInfo( job ) )
{
    int  task_mode       = TCMSUPER;
    long user_mode_stack = 0;
    long supv_mode_stack = stack_size;
    int  tid;
    int  err;

    tid = sc_tecreate( thread_run, VrtxAnyTaskId, prio,
		       task_mode, user_mode_stack, supv_mode_stack,
		       (char *)_info, VrtxJustPointer, &err );

    if( err != RET_OK )
	RTDiag::panic( "sc_tecreate" );
    else
	_info->tid = tid;
}

RTThreadInfo::RTThreadInfo( RTJob * _job )
    : sync(      )
    , job ( _job )
    , tid (    0 )
    , done(    0 )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( tid != 0 )
    {
	while( done == 0 )
	    sync.wait();

	int err;

	sc_tdelete( tid, 0, &err );

	if( err != RET_OK )
	    RTDiag::panic( "sc_tdelete" );
    }

#ifdef _lint
    job = (RTJob *)0;
#endif
}
